/****************************************************************************
**
** Copyright (C) 2013 Jolla Ltd.
** Contact: lorn.potter@jollamobile.com
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
****************************************************************************/

#include "qofonomanager.h"
#include "dbus/ofonomanager.h"
#include <QVariant>
#include <QTimer>
#include "dbustypes.h"

QDBusArgument &operator<<(QDBusArgument &argument, const ObjectPathProperties &modem)
{
    argument.beginStructure();
    argument << modem.path << modem.properties;
    argument.endStructure();
    return argument;
}

const QDBusArgument &operator>>(const QDBusArgument &argument, ObjectPathProperties &modem)
{
    argument.beginStructure();
    argument >> modem.path >> modem.properties;
    argument.endStructure();
    return argument;
}

class QOfonoManagerPrivate
{
public:
    QOfonoManagerPrivate();
    OfonoManager *ofonoManager;
    QStringList modems;
    bool available;
    QDBusServiceWatcher *ofonoWatcher;
};

QOfonoManagerPrivate::QOfonoManagerPrivate() :
 ofonoManager(0)
, modems(QStringList())
, available(0)
, ofonoWatcher(0)
{
    qDBusRegisterMetaType<ObjectPathProperties>();
    qDBusRegisterMetaType<ObjectPathPropertiesList>();
}

QOfonoManager::QOfonoManager(QObject *parent) :
    QObject(parent)
  , d_ptr(new QOfonoManagerPrivate)
{

    d_ptr->ofonoWatcher = new QDBusServiceWatcher("org.ofono",QDBusConnection::systemBus(),
            QDBusServiceWatcher::WatchForRegistration |
            QDBusServiceWatcher::WatchForUnregistration, this);

    connect(d_ptr->ofonoWatcher, SIGNAL(serviceRegistered(QString)),
            this, SLOT(connectToOfono(QString)));
    connect(d_ptr->ofonoWatcher, SIGNAL(serviceUnregistered(QString)),
            this, SLOT(ofonoUnregistered(QString)));

    d_ptr->available = QDBusConnection::systemBus().interface()->isServiceRegistered("org.ofono");
    if(d_ptr->available) {
        connectToOfono(QString());
    }
}

QOfonoManager::~QOfonoManager()
{
    delete d_ptr;
}

QStringList QOfonoManager::modems()
{
    return d_ptr->modems;
}

void QOfonoManager::onModemAdd(const QDBusObjectPath& path, const QVariantMap& /*map*/)
{
    QString pathStr = path.path();
    if (!d_ptr->modems.contains(pathStr)) {
        d_ptr->modems.append(pathStr);
        Q_EMIT modemAdded(pathStr);
        Q_EMIT modemsChanged(d_ptr->modems);
    }
}

void QOfonoManager::onModemRemove(const QDBusObjectPath& path)
{
    QString pathStr = path.path();
    if (!d_ptr->modems.contains(pathStr)) {
        d_ptr->modems.removeOne(pathStr);
        Q_EMIT modemRemoved(pathStr);
        Q_EMIT modemsChanged(d_ptr->modems);
    }
}

bool QOfonoManager::available() const
{
    return d_ptr->available;
}

void QOfonoManager::connectToOfono(const QString &)
{
    d_ptr->ofonoManager = new OfonoManager("org.ofono","/",QDBusConnection::systemBus(),this);
    if (d_ptr->ofonoManager->isValid()) {
        QDBusReply<ObjectPathPropertiesList> reply = d_ptr->ofonoManager->GetModems();
        foreach(ObjectPathProperties modem, reply.value()) {
            d_ptr->modems << modem.path.path();
            Q_EMIT modemAdded(modem.path.path());
        }
        Q_EMIT modemsChanged(d_ptr->modems);

    }
    Q_EMIT availableChanged(true);
}

void QOfonoManager::ofonoUnregistered(const QString &)
{
    d_ptr->available = false;
    delete d_ptr->ofonoManager;
    Q_FOREACH(const QString &modem, d_ptr->modems) {
        Q_EMIT modemRemoved(modem);
    }
    d_ptr->modems.clear();
    Q_EMIT modemsChanged(d_ptr->modems);
    Q_EMIT availableChanged(false);
}
